package br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertyDescriptor;

import br.edu.fei.dcc.raciocinioespacial.editor.application.Application;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ModelElement;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericPropertyTypeEnum;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.PropertyDescriptorComparer;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.TextProperty;

/**
 * Abstract prototype of a Shape.
 * Possui um tamanho (largura e altura), uma localizao (posio x e y) e uma lista das conexes de entrada
 * e sada. Use subclasses para inicializar uma forma especfica.
 */
public abstract class Shape extends ModelElement {
	public static final String Proprerty_Localization = "shape.localization";
	public static final String Proprerty_X = "shape.x";
	public static final String Proprerty_Y = "shape.y";
	public static final String Proprerty_Name = "shape.name";
	private static final long serialVersionUID = 1;
	protected static Image createImage(String name) {
		InputStream stream = Application.class.getResourceAsStream(name);
		Image image = new Image(null, stream);
		try {
			stream.close();
		} catch (IOException ioe) {
		}
		return image;
	}
	protected String name;
	protected Point localization;
	protected Dimension size;
	private ShapesDiagram parent;

	protected transient List<IPropertyDescriptor> descriptors;
	
	public Shape()
	{
		this.inicia();
		this.criaPropriedades();
	}

	protected void criaPropriedades() {
		this.getDescriptors().add(new TextProperty(Proprerty_Name, "Name"));
		this.getDescriptors().add(new NumericProperty(Proprerty_X, "X", NumericPropertyTypeEnum.Integer));
		this.getDescriptors().add(new NumericProperty(Proprerty_Y, "Y", NumericPropertyTypeEnum.Integer));
	}

	public List<IPropertyDescriptor> getDescriptors() {
		if(this.descriptors==null)
			this.setDescriptors(new ArrayList<IPropertyDescriptor>());
		return this.descriptors;
	}
	
	/**
	 * Return an icon for this ModelElement.
	 * @return An image 16x16 or null.
	 */
	public abstract Image getIcon();

	/**
	 * Return the localization of this Shape.
	 */
	public Point getLocalization() {
		return this.localization.getCopy();
	}

	/**
	 * Return the name typed to each object
	 */
	public String getName() {
		return name;
	}

	/**
	 * Return the parent ShapeDiagram
	 */
	public ShapesDiagram getParent() {
		return parent;
	}

	/**
	 * Return an array of IPropertyDescriptors for this Shape.
	 * @see #descriptors
	 * @see #getPropertyValue(Object)
	 * @see #setPropertyValue(Object, Object)
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] retorno = new IPropertyDescriptor[this.getDescriptors().size()];
		Collections.sort(this.getDescriptors(), new PropertyDescriptorComparer());
		this.getDescriptors().toArray(retorno);
		return retorno;
	}

	/**
	 * Return the property value of the propertyId requested, or null.
	 * <p>A viso de propriedades usa os identificadores a partir do array de IPropertyDescriptors
	 * para obter o valor das propriedades correspondentes.</p>
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public Object getPropertyValue(Object propertyId) {
		if (Proprerty_X.equals(propertyId)) {
			return Integer.toString(this.getLocalization().x);
		}
		if (Proprerty_Y.equals(propertyId)) {
			return Integer.toString(this.getLocalization().y);
		}
		if (Proprerty_Name.equals(propertyId)) {
			return this.getName();
		}
		return super.getPropertyValue(propertyId);
	}
	
	/**
	 * Return the size of this Shape.
	 */
	public Dimension getSize() {
		return this.size.getCopy();
	}

	/**
	 * Starts all properties of this object
	 */
	protected void inicia() {
		this.localization = new Point(10, 10);
		this.size = new Dimension(5, 5);
	}

	public void setDescriptors(List<IPropertyDescriptor> descriptors) {
		this.descriptors = descriptors;
	}

	/**
	 * Define the localization of this Shape.
	 */
	public void setLocalization(Point newLocation) {
		if (newLocation == null) {
			throw new IllegalArgumentException();
		}
		this.localization.setLocation(newLocation);
		firePropertyChange(Proprerty_Localization, null, localization);
	}
	
	/**
	 * Set the name for this object
	 */
	public void setNome(String name) {
		if(!this.getParent().existName(name))
		{
			String oldName = this.name;
			this.name = name;
			firePropertyChange(Proprerty_Name, oldName, this.name);
		}
	}
	
	public void setParent(ShapesDiagram parent) {
		this.parent = parent;
	}

	/**
	 * Define o valor de uma propriedade com o identificador fornecido.
	 * Se no forem encontradas propriedades com essa identificao, a chamada  passada para a superclasse.
	 * <p>A viso de propriedade usa os identificadores a partir do array de IPropertyDescriptors para definir os valores.
	 * das respectivas propriedades.</p>
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public void setPropertyValue(Object propertyId, Object value) {
		if (Proprerty_X.equals(propertyId)) {
			int x = Integer.parseInt((String) value);
			this.setLocalization(new Point(x, localization.y));
		}
		else if (Proprerty_Y.equals(propertyId)) {
			int y = Integer.parseInt((String) value);
			this.setLocalization(new Point(localization.x, y));
		}
		else if (Proprerty_Name.equals(propertyId)) {
			this.setNome((String)value);
		}
		else {
			super.setPropertyValue(propertyId, value);
		}
	}

	/**
	 * Define the size of this shape.
	 */
	public void setSize(Dimension newSize) {
		size = newSize;
	}
}
