package br.edu.fei.dcc.raciocinioespacial.editor.shapes.model;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

/**
 * Abstract prototype of our elements model.
 * <p>This class gives the needed resources for all elements that may be created.</p>
 */
public abstract class ModelElement implements IPropertySource, Serializable {
	/** Void property for description. */
	private static final IPropertyDescriptor[] EMPTY_ARRAY = new IPropertyDescriptor[0];

	private static final long serialVersionUID = 1;
	
	/** Delegate used to implement the base method for properties modifications. */
	private transient PropertyChangeSupport pcsDelegate = new PropertyChangeSupport(this);

	/** 
	 * Add a PropertyChangeListener not null for the object.
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}

	/** 
	 * Report an property alteration to listeners (for example edit parts).
	 */
	protected void firePropertyChange(String property, Object oldValue, Object newValue) {
		if (pcsDelegate.hasListeners(property)) {
			pcsDelegate.firePropertyChange(property, oldValue, newValue);
		}
	}

	/**
	 * Return itself!
	 */
	public Object getEditableValue() {
		return this;
	}

	/**
	 * Classes, which will heritage from this class, must implement this method.
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return EMPTY_ARRAY;
	}

	/**
	 * Classes, which will heritage from this class, must implement this method.
	 */
	public Object getPropertyValue(Object id) {
		return null;
	}

	/**
	 * Classes, which will heritage from this class, must implement this method.
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/** 
	 * Unserialization constructor.
	 * @see java.io.Serializable
	 */
	private void readObject(ObjectInputStream in) 
	throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		pcsDelegate = new PropertyChangeSupport(this);
	}

	/** 
	 * Remove a PropertyChangeListener.
	 */
	public synchronized void removePropertyChangeListener(PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}

	/**
	 * Classes, which will heritage from this class, must implement this method.
	 */
	public void resetPropertyValue(Object id) {
	}

	/**
	 * Classes, which will heritage from this class, must implement this method.
	 */
	public void setPropertyValue(Object id, Object value) {
	}
}
