package br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Mobile2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;

/**
 * Comando para mover e redimensionar as formas/shapes.
 * O comando pode ser desfeito e refeito.
 */
public class ShapeSetConstraintCommand extends Command {
	/** Armazena o novo tamanho e a nova localizao. */
	private final Rectangle newBounds;
	/** Armazena o antigo tamanho e a antiga localizao. */
	private Rectangle oldBounds;
	/** Requisio para mover ou redimensionar o objeto. */
	private final ChangeBoundsRequest request;

	/** Forma/Shape a ser manipulada. */
	private final Shape shape;

	/**
	 * Cria um comando que redimensiona ou move uma forma/shape. 
	 * @param shape		a forma/shape a ser redimensionado.
	 * @param req		quem requisita a movimentao ou o redimensionamento.
	 * @param newBounds novo tamanho e nova localizao.
	 * @throws IllegalArgumentException se algum parmetro for nulo.
	 */
	public ShapeSetConstraintCommand(Shape shape, ChangeBoundsRequest req, 
			Rectangle newBounds) {
		if (shape == null || req == null || newBounds == null) {
			throw new IllegalArgumentException();
		}
		this.shape = shape;
		this.request = req;
		
		// If the shape is a point, we force the perpendicular position according to parent shape
		if (shape instanceof Ponto2DShape)
		{
			Mobile2DShape mobDad = ((Ponto2DShape)shape).getPai();
			Point ptReferece = mobDad.getLocalization();
			
			if (((Ponto2DShape)shape).getInstant() > 0)
				ptReferece = mobDad.getPoints().get(((Ponto2DShape)shape).getInstant() - 1).getLocalization();
			
			if (Math.abs(ptReferece.x - newBounds.x) < Math.abs(ptReferece.y - newBounds.y))
				newBounds.x = ptReferece.x;
			else
				newBounds.y = ptReferece.y;
		}
		
		this.newBounds = new Rectangle(newBounds.x, newBounds.y, shape.getSize().width, shape.getSize().height);
		setLabel("Mover / Redimensionar");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		Object type = request.getType();
		// Verifica se o requisitante realmente  suportado.
		return (RequestConstants.REQ_MOVE.equals(type)
				|| RequestConstants.REQ_MOVE_CHILDREN.equals(type) 
				|| RequestConstants.REQ_RESIZE.equals(type)
				|| RequestConstants.REQ_RESIZE_CHILDREN.equals(type));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		oldBounds = new Rectangle(shape.getLocalization(), shape.getSize());
		redo();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		shape.setSize(newBounds.getSize());
		shape.setLocalization(newBounds.getLocation());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		shape.setSize(oldBounds.getSize());
		shape.setLocalization(oldBounds.getLocation());
	}
}